# sb::Admin::Profile
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::Admin::Profile;
use strict;
use sb::Admin::User (); # for category_selector (in List) / check_user (in User)
use base qw( sb::Admin::User );
use vars qw( $VERSION );
$VERSION = '0.06';
# 0.06 [2008/10/29] modified _update_info to update a notice flag
# 0.05 [2007/11/05] added title_name
# 0.04 [2007/09/26] enabled $self->conf->save
# 0.03 [2007/09/26] modified _update_basic to update default category correctly
# 0.02 [2007/09/12] ported from Serene Bach 2.* (merged Profile and Editor)
# 0.01 [2005/06/08] chnaged _update_profile to update correctly
# 0.00 [2005/04/09] generate
# --------------------------------------
# // defines
sub TEMPLATE     (){ 'profile.html' };
sub JSFILE_NAME  (){ 'script/adminutils.js' }
sub MAX_LEVEL    (){ 2 };
sub DEFAULT_TYPE (){ 'info' };
my %mProfileType = (
	'info'   => {'submit'=>1,},
	'basic'  => {'submit'=>1,},
	'editor' => {'submit'=>1,},
);
# --------------------------------------
# // override base class
sub js_paths
{
	my $self = shift;
	my $editor = $self->get_editor();
	return( $self->static_path . JSFILE_NAME, $editor->javascripts() );
}
sub title_name
{
	my $self = shift;
	my $title = $self->SUPER::title_name();
	return $title if ($title);
	$title = sb->string(' &gt; Profile');
	$title .= sb->string(
		  ($self->{'type'} eq 'basic')  ? ' &gt; Settings'
		: ($self->{'type'} eq 'editor') ? ' &gt; Editor settings'
		:                                 '');
	return $title;
}
# --------------------------------------
# // public functions
sub callback
{
	my $self = shift;
	$self->{'type'} = $self->cgi->value('__type');
	$self->{'type'} = DEFAULT_TYPE if (!$mProfileType{$self->{'type'}});
	$self->{'target'} = $self->user;
	if ($self->cgi->value('pid') =~ /\d+/ and $self->users->{$self->cgi->value('pid')})
	{
		$self->{'target'} = $self->users->{$self->cgi->value('pid')};
	}
	return ($self->regi)
		? $self->_update(@_)
		: $self->_display(@_);
}
# --------------------------------------
# // private functions
sub _update
{
	my $self = shift;
	my $msg = '';
	die(sb->string('Unknown error.') . "\n") if (!$self->{'target'});
	SWITCH_TYPE: {
		$_ = $self->{'type'};
		/^info$/ && do
		{
			$msg = $self->_update_info();
			last SWITCH_TYPE;
		};
		/^basic$/ && do
		{
			$msg = $self->_update_basic();
			last SWITCH_TYPE;
		};
		/^editor$/ && do
		{
			$msg = $self->_update_editor();
			last SWITCH_TYPE;
		};
	} # end of SWITCH_TYPE
	return $self->_display('message'=>$msg);
}
sub _display
{
	my $self = shift;
	my %param = (
		'message' => undef,
		@_
	);
	my $temp = sb::Template->new($self->load_template('file'=>TEMPLATE));
	SWITCH_TYPE:
	{
		$_ = $self->{'type'};
		/^info$/ && do
		{
			$self->_display_info($temp);
			last SWITCH_TYPE;
		};
		/^basic$/ && do
		{
			$self->_display_basic($temp);
			last SWITCH_TYPE;
		};
		/^editor$/ && do
		{
			$self->_display_editor($temp);
			last SWITCH_TYPE;
		};
	} # end of SWITCH_TYPE
	$temp->num(0);
	$temp->tag('sb_selected_type' => $self->{'type'});
	$temp->tag('sb_profile_menu_' . $self->{'type'} => ' class="current"');
	$temp->tag('sb_profile_pid' => $self->{'target'}->id);
	$temp->tag('sb_profile_title' => $self->{'target'}->real);
	$temp->block('sb_profile_submit' => $mProfileType{$self->{'type'}}->{'submit'});
	$temp->block('sb_profile_' . $self->{'type'} => 1);
	$self->common_parts($temp);
	$self->set_message($temp,$param{'message'});
	return $self->set_main($temp->output)
}
# --------------------------------------
# // private functions - update
sub _update_info
{
	my $self = shift;
	my $user = $self->{'target'};
	my $name_changed = ($self->cgi->value('user_name') ne $user->name);
	my $pass_changed = ($self->cgi->value('user_pass') ne '');
	my $msg = $self->check_user(
		'name'       => $self->cgi->value('user_name'),
		'pass'       => $self->cgi->value('user_pass'),
		'conf'       => $self->cgi->value('user_passconf'),
		'check_pass' => $pass_changed,
		'check_name' => $name_changed,
	);
	return $msg if ($msg ne '');
	$user->name($self->cgi->value('user_name')) if ( $name_changed );
	$user->pass($self->cgi->value('user_pass')) if ( $pass_changed );
	$user->real(sb::Text->new($self->cgi->value('user_real'))->entitized());
	$user->mail(sb::Text->new($self->cgi->value('user_mail'))->entitized());
	$user->aws(sb::Text->new($self->cgi->value('user_aws'))->entitized());
	$user->prof($self->cgi->value('user_prof'));
	$user->disp($self->cgi->value('user_displist'));
	$user->form($self->cgi->value('user_prof_breaks'));
	$user->notice(int($self->cgi->value('profile_notice')));
	if ($user->id != 0 and $self->cgi->value('user_stat') ne '')
	{
		my $level = int($self->cgi->value('user_stat'));
		$level = 2 if ($level <= 0 or $level > MAX_LEVEL);
		$user->stat($level);
	}
	my $check = sb::Data->update($user);
	if ($user->id == $self->{'user'}->id)
	{
		$self->user($user);
		$self->users->{$user->id} = $user;
		$msg .= sb->string('User ID has been changed.  Please re-log in.<br />') if ($name_changed and $check);
	}
	$self->build_list('user_list');
	return ($check) 
		? $msg . sb->string('Modified.<br />')
		: $msg . sb->string('Failed to update database.');
}
sub _update_basic
{
	my $self = shift;
	my $user = $self->{'target'};
	foreach my $key ($user->get_option_keys)
	{
		next if ($key =~ /^(format)|(status)|(edit_tool)|(imagelist)|(imagemax)$/);
		$user->set_option($key => int($self->cgi->value('profile_' . $key)));
	}
	$user->set_option('format'=>$self->_encode_text($self->cgi->value('profile_format')));
	$user->cat(($self->cgi->value('profile_category') =~ /^\d$/) ? $self->cgi->value('profile_category') : -1);
	$user->ad_css($self->cgi->value('profile_admin_style'));
	if ($self->cgi->value('update_ping_check'))
	{
		$self->conf->value('conf_edit_ping' => $self->cgi->value('profile_pinglist'));
		$self->conf->save($self->conf->config_file());
	}
	return (sb::Data->update($user))
		? sb->string('Modified.<br />')
		: sb->string('Failed to update database.');
}
sub _update_editor
{
	my $self = shift;
	my $editor = $self->get_editor()->new('user' => $self->{'target'});
	return ($editor->update_option())
		? sb->string('Modified.<br />')
		: sb->string('Failed to update database.');
}
# --------------------------------------
# // private functions - display
sub _display_info
{
	my $self = shift;
	my $temp = shift;
	my $user = $self->{'target'};
	my $editor = $self->get_editor()->new('user' => $self->user);
	$temp->num(0);
	$temp->tag('sb_profile_editor' => $editor->display_editor(
		'id' => 'user_prof',
		'content' => sb::Text->new($user->prof)->entitized(),
	));
	$temp->tag('sb_profile_name' => $user->name);
	$temp->tag('sb_profile_real' => $user->real);
	$temp->tag('sb_profile_mail' => $user->mail);
	$temp->tag('sb_profile_aws'  => $user->aws); 
	$temp->tag('sb_profile_form' => ($user->form ne '') ? 'checked="checked"' : '');
	$temp->tag('sb_profile_displist_' . $user->disp => 'selected="selected"');
	# [note] super user (the first creator) is always user id == 0
	if ( $user == $self->user->id 
	  or $self->user->stat == 0)
	{
		$temp->block('sb_profile_changepass'=>1);
	}
	if (  $self->user->stat == 0 
	  and $user->id != 0)
	{
		$temp->num(0);
		$temp->tag('sb_profile_stat_' . $user->stat => 'selected="selected"');
		$temp->block('sb_profile_permission'=>1);
	}
	my $weblog = sb::Data->load('Weblog','id'=>0);
	if ($weblog->stype ne '')
	{
		$temp->num(0);
		$temp->tag('sb_profile_notice_1'=>'selected="selected"') if ($user->notice);
		$temp->block('sb_profile_notice'=>1);
	}
	$temp->block('sb_profile_sbit' => 1);
	return;
}
sub _display_basic
{
	my $self = shift;
	my $temp = shift;
	my $user = $self->{'target'};
	my %option;
	foreach my $key ($user->get_option_keys)
	{
		next if ($key =~ /^(status)|(edit_tool)|(imagelist)|(imagemax)$/);
		$option{$key} = $user->get_option($key);
	}
	{ # text filter
		my $selector = '';
		my @filters  = sb::Plugin->get->text_filters();
		foreach my $name (@filters)
		{
			$selector .= '<option value="' . $name . '"';
			$selector .= ' selected="selected"' if ($option{'format'} eq $name);
			$selector .= '>' . $name . '</option>';
		}
		$temp->num(0);
		$temp->tag('sb_profile_extra_format'=>$selector);
	}
	{ # default category
		$self->{'cat'} = { sb::Data->load_as_hash('Category') };
		$temp->tag('sb_profile_category'=>
			'<option value="none">' . sb->string('No category') . '</option>' .
			$self->category_selector(
				'cat' => [ sort { $b->order <=> $a->order } values(%{$self->{'cat'}}) ],
				'now' => $user->cat,
			)
		);
	}
	{ # options
		$temp->num(0);
		foreach my $key (keys(%option))
		{
			next if ($option{$key} !~ /^\d+$/);
			$temp->tag('sb_profile_' . $key . '_' . $option{$key} => 'selected="selected"');
		}
	}
	{ # extra style
		my $selector = '';
		foreach my $style ($self->extra_styleset)
		{
			$selector .= '<option value="' . $style . '"';
			$selector .= ' selected="selected"' if ($user->ad_css eq $style);
			$selector .= '>' . $style . '</option>';
		}
		$temp->num(0);
		$temp->tag('sb_profile_styleset'=>$selector);
	}
	if (  $self->user->stat == 0 
	  and $user->id == $self->user->id)
	{ # update pings settings
		my @ping_list = split('\\n',$self->conf->value('conf_edit_ping'));
		$temp->num(0);
		$temp->tag('sb_profile_pinglist'=>join("\n",@ping_list));
		$temp->block('sb_profile_admin'=>1);
	}
	return;
}
sub _display_editor
{
	my $self = shift;
	my $temp = shift;
	my $editor = $self->get_editor()->new('user' => $self->{'target'});
	$temp->num(0);
	$temp->tag('sb_profile_editor_option' => $editor->display_option());
	return;
}
# --------------------------------------
# // private functions - utilities
sub _encode_text
{
	my $self = shift;
	my $text = shift;
	$text =~ s/:/&#58;/g;
	$text =~ s/\[/&#91;/g;
	$text =~ s/\]/&#93;/g;
	$text =~ s/\'/&#39;/g;
	return $text;
}
1;
__END__
