// spica javascript libraries - string.js
// == written by Takuya Otani <takuya.otani@gmail.com> ===
// == Copyright (C) 2006 SimpleBoxes/SerendipityNZ Ltd. ==

// [guideline]
// -- http://serennz.sakura.ne.jp/guidelines/js_en_000.txt
// -- http://serennz.sakura.ne.jp/guidelines/js_ja_000.txt

// ver 0.00 [2008/11/01] the first attempt

Spica.SelectedText = function(object)
{
	var self = this;
	self.obj = object; // target object (textarea, input)
	self.bgn = null;
	self.end = null;
	self.range = null;
	self.selection = null;
	self._node = null;
	if (self.obj) self.init();
	return self;
}
Spica.SelectedText.prototype = {
	init : function()
	{ // initialize
		var self = this;
		if (document.selection && !Spica.Browser.isOpera)
		{
			self.obj.focus();
			self.range = document.selection.createRange()
			// a hack to check whether strings in textarea/input are selected or not.
			if (self.range.queryCommandEnabled && self.range.queryCommandEnabled("Bold"))
				self.range = null;
		}
		else if (self.obj.selectionEnd != undefined && self.obj.selectionStart != undefined)
		{
			self.bgn = self.obj.selectionStart;
			self.end = self.obj.selectionEnd;
		}
		self.selection = window.getSelection();
	},
	get_text : function()
	{
		var self = this;
		if (self.range)
			return self.range.text;
		else if (self.bgn != null && self.end != null)
			return self.obj.value.substring(self.bgn, self.end);
		else if (self.selection && self.selection.toString())
			return self.selection.toString();
		else if (self.selection)
			return self.selection;
		else
			return '';
	},
	set_text : function(str)
	{
		var self = this;
		if (!self.obj) return;
		if (self.range)
		{
			self.range.text = str;
			self.range.moveStart("character", -str.length);
		}
		else if (self.bgn != null && self.end != null)
		{
			var forward  = self.obj.value.substring(0, self.bgn);
			var backward = self.obj.value.substring(self.end, self.obj.value.length);
			self.obj.value = forward + str.toString() + backward;
			self.end = self.bgn + str.length;
		}
		else if (self.selection)
		{
			var range = self._get_range();
			var node = window.document.createTextNode(str);
			range.deleteContents();
			range.insertNode(node);
			self._node = node;
		}
		else
		{
			self.obj.value = self.obj.value + str.toString();
		}
	},
	select : function()
	{
		var self = this;
		if (!self.obj) return;
		if (self.range)
			self.range.select();
		else if (self.bgn != null && self.end != null)
			self.obj.setSelectionRange(self.bgn, self.end);
		else if (!self.selection || !self._node);
			return;
		var range = self._get_range();
		range.selectNode(self._node);
	},
	escape : function(mark)
	{
		var self = this;
		var str = self.get_text();
		var escaped = '\\' + mark;
		str = str.split(mark).join(escaped);
		self.set_text(str);
	},
	clipon : function(start,end)
	{
		var self = this;
		var str = self.get_text();
		var line = str.match(/\r?\n$|\r$/);
		str = str.replace(/\r?\n$|\r$/,''); // remove linefeed at the end of line
		str = str.replace(/\r?\n|\r/g,'<br />'); // replace rest of linefeeds
		str = start + str + end;
		if (line) str = str + line;
		self.set_text(str);
		self.select();
	},
	mark_line : function(mark)
	{
		var self = this;
		var str = mark + self.get_text(); // insert mark at the beginning of selected text
		str = str.replace(/(\r?\n|\r)/g,"$1" + mark); // add mark at all linefeeds
		self.set_text(str);
		self.select();
	},
	_get_range : function()
	{
		if (!self.selection) return;
		var selection = self.selection;
		if (selection.getRangeAt) return selection.getRangeAt(0);
		var range = window.document.createRange();
		range.setStart(selection.baseNode, selection.baseOffset);
		range.setEnd(selection.extentNode, selection.extentOffset);
		return range;
	}
};
String.prototype.entitize = function()
{
	var str = "" + this;
	str = str.split("&").join("&amp;");
	str = str.split("<").join("&lt;");
	str = str.split(">").join("&gt;");
	str = str.split('"').join("&quot;");
	str = str.split("{").join("&#123;");
	str = str.split("}").join("&#125;");
	str = str.split("'").join("&#039;");
	return str;
}
String.prototype.detitize = function()
{
	var str = "" + this;
	str = str.split("&#039;").join("'");
	str = str.split('&#123;').join("{");
	str = str.split('&#125;').join("}");
	str = str.split("&quot;").join('"');
	str = str.split("&gt;").join(">");
	str = str.split("&lt;").join("<");
	str = str.split("&amp;").join("&");
	return str;
}
String.prototype.printf = function()
{
	var str = "" + this;
	var param = arguments, i = 0;
	return str.replace(/%(%|s)/g, function(){ return (RegExp.$1 == 's') ? param[i++] : '%'; });
}
